NXTSensor subclass: #NXTColorSensor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Enchanting-NXT Peripherals'!

!NXTColorSensor methodsFor: 'as yet unclassified' stamp: 'CWB 9/29/2011 06:54'!
colorList
"Returns the list of colors recognized by this device"

	(concreteType = 'NXT Color Sensor') ifTrue: [
		^ self colorsForLEGOSensor.
	].
	(concreteType = 'HiTechnic Color Sensor') ifTrue: [
		^ self colorsForHiTechnicSensor.
	]. 
	^ self error: 'Unknown color sensor type'.

! !

!NXTColorSensor methodsFor: 'as yet unclassified' stamp: 'CWB 9/29/2011 07:02'!
colorsForHiTechnicSensor
"Returns the list of colors seen by the HiTechnic device (as measured from the graphic for the NXT-G block)"

	^ {
		Color r: 0 		g: 0 		b: 0			"black" .

		Color r: 0.514 	g: 0 		b: 0.8		"purple" .
		Color r: 0.059 	g: 0 		b: 0.8		"blue" .
		Color r: 0	 	g: 0.533 	b: 0.8		"teal" .
		Color r: 0	 	g: 0.8 		b: 0.173		"green" .
		Color r: 0.592 	g: 0.8 		b: 0			"greenish yellow" .
		Color r: 1 		g: 1 		b: 0			"yellow" .
		Color r: 1	 	g: 0.757 	b: 0			"yellowish orange" .
		Color r: 1	 	g: 0.478 	b: 0.04		"orange" .
		Color r: 1	 	g: 0 		b: 0			"red" .
		Color r: 1 		g: 0.09 		b: 0.698		"reddish purple" .

		Color r: 0.902 	g: 0.725		b: 1			"light purple" .
		Color r: 0.745 	g: 0.725 	b: 1			"light blue" .
		Color r: 0.725 	g: 1 		b: 0.784		"light green" .
		Color r: 1 		g: 1 		b: 0.725		"light yellow" .
		Color r: 1	 	g: 0.725 	b: 0.725		"light red" .
		Color r: 1		g: 0.725 	b: 0.91		"light pinkish purple" .

		Color r: 1	 	g: 1 		b: 1			"white" .
	}.

! !

!NXTColorSensor methodsFor: 'as yet unclassified' stamp: 'CWB 9/29/2011 06:51'!
colorsForLEGOSensor
"Returns the list of colors seen by the HiTechnic device (as measured from the graphic for the NXT-G block)"

	^ {
		Color black .
		Color blue .
		Color green .
		Color yellow .
		Color red . 
		Color white 
	}! !

!NXTColorSensor methodsFor: 'as yet unclassified' stamp: 'CWB 9/30/2011 22:17'!
customExportColor: chosenColor onStream: aStream

"Take the color, and map it onto the closest color that this sensor can quickly sense."

	| colorList colorIndex index minDistance |

	colorList _ self colorList.

	"Find the closest color"
	index _ 0.	"Note that squeak indices start at 1, but the emitted color index starts at 0."
	minDistance _ 2.	"max possible distance is 1.0"
	colorList do: [ :color |
		| distance |
		distance _ color distanceFrom: chosenColor.
		(distance < minDistance) ifTrue: [
			minDistance _ distance.
			colorIndex _ index.
		].
		index _ index + 1.
	].

	aStream nextPutAll: colorIndex asString.
! !

!NXTColorSensor methodsFor: 'as yet unclassified' stamp: 'JM 8/23/2011 11:04'!
emitInitializationCodeOn: aStream for: anExporter
"Outputs the code to initialize NXT peripherals."

	| sensorClassName  |

	sensorClassName _ anExporter codeSpecFrom: concreteType asString.

	anExporter indentOn: aStream.
	aStream nextPutAll: 'public static final ', 
		sensorClassName, ' ', self variableName, ' = new ',
		sensorClassName, '(', self portName , ');'; cr.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NXTColorSensor class
	instanceVariableNames: ''!

!NXTColorSensor class methodsFor: 'accessing' stamp: 'JM 8/5/2011 12:40'!
baseName

	^ 'color sensor'! !

!NXTColorSensor class methodsFor: 'accessing' stamp: 'JM 8/22/2011 16:35'!
concreteTypes
"Returns a list of physical devices that implement the functions of this class."


	^ #(
		'NXT Color Sensor'
		'HiTechnic Color Sensor'
	).! !


!NXTColorSensor class methodsFor: 'block specs' stamp: 'JM 8/5/2011 12:41'!
configBlockSpec

	^ 'Measures color using a %p named %s.'.! !

!NXTColorSensor class methodsFor: 'block specs' stamp: 'CWB 9/28/2011 13:34'!
providesBlocks
"Returns a list of block specifications to be used in the main user interface if there is any device configured that can use these blocks.  Note that the 'selectors' are all unique, start with the class name followed by a colon, but do not actually have functions that go with them."

	| blocks |
	
	blocks _ #(
		'sensing'
			('%P %J' 						r	NXTColorSensor:sensor:param:)
			('%P sees %c' 					b	NXTColorSensor:seesColor:)
		).

	^ blocks  ! !


!NXTColorSensor class methodsFor: 'dropdown lists' stamp: 'JM 8/23/2011 10:32'!
customListNumber: aNumber
"Peripherals can return custom lists (or nil)."

	(aNumber = 1) ifTrue: [
		"Things that can be read from the sensor or its settings"
		^ #('red value' 'green value' 'blue value' 'color id')	
	].

	^ nil.! !
