NXTSensor subclass: #NXTLightDetector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Enchanting-NXT Peripherals'!
!NXTLightDetector commentStamp: '<historical>' prior: 0!
For light sensors (and color sensors) that also have lamps (LEDs that turn on and off).  This is actually like the LampLightDetector interface in LeJOS, not, as the name suggests, like LightDetector.!


!NXTLightDetector methodsFor: 'as yet unclassified' stamp: 'CWB 6/10/2011 15:30'!
emitInitializationCodeOn: aStream for: anExporter
"Outputs the code to initialize NXT peripherals."

"Output should look something like:
	public static final LightSensor S1 = new LightSensor(SensorPort.S1);
or 	public static final ColorLightSensor COLOR_LIGHT_SENSOR = new ColorLightSensor(SensorPort.S2,ColorLightSensor.TYPE_COLORRED);"

	| sensorClassName  |

	(concreteType = 'NXT Color Sensor') ifFalse: 
		[ ^ super emitInitializationCodeOn: aStream for: anExporter ].

	sensorClassName _ anExporter codeSpecFrom: concreteType asString.

	anExporter indentOn: aStream.
	aStream nextPutAll: 'public static final ', 
		sensorClassName, ' ', self variableName, ' = new ',
		sensorClassName, '(', self portName, 
		', ColorSensor.TYPE_COLORRED);'; cr.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NXTLightDetector class
	instanceVariableNames: ''!

!NXTLightDetector class methodsFor: 'accessing' stamp: 'CWB 11/20/2010 20:21'!
baseName

	^ 'light sensor'! !

!NXTLightDetector class methodsFor: 'accessing' stamp: 'CWB 12/18/2010 11:07'!
concreteTypes
"Returns a list of physical devices that implement the functions of this class."


	^ #(
		'NXT Light Sensor'
		'NXT Color Sensor'
		'RCX Light Sensor'
	).! !


!NXTLightDetector class methodsFor: 'block specs' stamp: 'CWB 1/3/2011 14:39'!
configBlockSpec

	^ 'Measures brightness using a %p named %s.'.! !

!NXTLightDetector class methodsFor: 'block specs' stamp: 'CWB 1/3/2011 14:48'!
providesBlocks
"Returns a list of block specifications to be used in the main user interface if there is any device configured that can use these blocks.  Note that the 'selectors' are all unique, start with the class name followed by a colon, but do not actually have functions that go with them."

	| blocks |
	
	blocks _ #(
		'sensing'
			('%P sees light?' 				b	NXTLightDetector:seesLight:)
			('%P %J' 						r	NXTLightDetector:sensor:param:)
			('turn %P''s lamp %Q'			-	NXTLightDetector:setLightColor:)
			('%P set %K to %n' 				-	NXTLightDetector:sensor:param:set:)
			'-'
			('start auto-calibrating %P'		-	NXTLightDetector:startAutoCalibration:)
			('auto-calibrate %P to include current reading'	-	NXTLightDetector:addReadingToCalibration)
		).

	^ blocks  ! !


!NXTLightDetector class methodsFor: 'dropdown lists' stamp: 'CWB 1/3/2011 14:46'!
customListNumber: aNumber
"Peripherals can return custom lists (or nil)."

	(aNumber = 1) ifTrue: [
		"Things that can be read from the sensor or its settings"
		^ #('brightness' 'raw value' 'darkest point' 'brightest point')	
	].
	(aNumber = 2) ifTrue: [
		"Things you can set on the sensor"
		^ #('darkest point' 'brightest point')	
	].
	(aNumber = 3) ifTrue: [
		"Turn the lamp off or on (possibly with a color)"
		^ #('on (red)' 'on (green)' 'on (blue)' 'off (ambient)')	
	].

	^ nil.! !
